//Common Module
#include <SPI.h>  
//Ethenet Module
#include <Ethernet.h>
#include <ICMPPing.h>
//SD Module
#include <SD.h>
//LCD Module
#include <UTFT.h>
//Touch Module
//#include <ITDB02_Touch.h>
//DS1307 Module
#include <DS1307.h>

#define T_CLK 6
#define T_CS 5
#define T_DIN 48
#define T_DOUT 3
#define T_IRQ 2

#define X_CONST 240
#define Y_CONST 320

#define PREC_TOUCH_CONST 10

#define PixSizeX	13.78
#define PixOffsX	411

#define PixSizeY	11.01
#define PixOffsY	378

int TP_X,TP_Y;

extern uint8_t SmallFont[];
UTFT myGLCD(ITDB28,38,39,40,41);
//ITDB02_Touch  myTouch(6,5,48,3,2);

File myFile;

DS1307  rtc(20, 21);
Time  t;

byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED}; // max address for ethernet shield
byte ip[] = {10,0,0,21}; // ip address for ethernet shield
byte pingAddr[] = {10,0,0,20}; // ip address to ping

SOCKET pingSocket = 0;

char buffer [256];


void setup()
{
  unsigned int  i,j;
  unsigned char NextFlag = 0;
  char temp = 0;
  
  //Init serial
  Serial.begin(9600);
  //Init LCD
  myGLCD.InitLCD();
  myGLCD.setFont(SmallFont);
  //Init Touch
//  myTouch.InitTouch(LANDSCAPE);
//  myTouch.setPrecision(PREC_MEDIUM);
	Touch_Init();
  //Clean screen
  myGLCD.setBackColor(0, 0, 255);
  //Init Ethernet

  
  
  
  ////Test SD
  myGLCD.clrScr();
  myGLCD.print("Press the screen to start SD Test", LEFT, 0);
  Serial.print("Press the screen to start SD Test");
  while(!NextFlag)
  {
    if (Touch_DataAvailable())
    {
      Touch_Read();
      i=Touch_GetX();
      j=Touch_GetY();
      if((i > 0) && (i < 239) && (j > 0) && (j < 319))
        NextFlag = 1;
    }
  } 
    if (!SD.begin(4))
    {
      myGLCD.print("initialization failed!", LEFT, 16*1);
      Serial.println("initialization failed!");
      while(1);
    }
    Serial.println("initialization done.");
    myGLCD.print("initialization done.", LEFT, 16*2);
    myFile = SD.open("test.txt", FILE_WRITE);
	//Write a test file
    if (myFile) 
    {
      myGLCD.print("Writing to test.txt...", LEFT, 16*2);
      Serial.print("Writing to test.txt...");
      myFile.println("0123456789");
	// close the file:
      myFile.close();
      myGLCD.print("done.", LEFT, 16*3);
      Serial.println("done.");
    }
    else 
    {
      myGLCD.print("error opening test.txt", LEFT, 16*2);
    // if the file didn't open, print an error:
    Serial.println("error opening test.txt");
    }
	
	//Read test file
	myFile = SD.open("test.txt");
	if (myFile)
	{
		Serial.println("test.txt:");
		myGLCD.print("Read test file", LEFT, 16*4);
		// read from the file until there's nothing else in it:
		while (!myFile.available());
		for(i = 0;i<10;i++)
		{
			temp = myFile.read() - 48;
			if(temp != i)
			{
				myGLCD.print("Read test file Error", LEFT, 16*5);
				while(1);
			}
		}	
		// close the file:
		myFile.close();
	}
	else 
	{
		myGLCD.print("Read test file Error", LEFT, 16*6);
		// if the file didn't open, print an error:
		Serial.println("error opening test.txt");
	}
	
	
	
	
	//Test DS1307
	NextFlag = 0;
	myGLCD.print("Press the screen to start RTC Test", LEFT, 16*7);
	Serial.print("Press the screen to start RTC Test");
	myGLCD.clrScr();
	while(!NextFlag)
	{
		if (Touch_DataAvailable())
		{
		Touch_Read();
		i=Touch_GetX();
		j=Touch_GetY();
		if((i > 0) && (i < 239) && (j > 0) && (j < 319))
			NextFlag = 1;
		}
	}
	rtc.halt(false);
	
	rtc.setDOW(SUNDAY);        // Set Day-of-Week to SUNDAY
	rtc.setTime(12, 0, 0);     // Set the time to 12:00:00 (24hr format)
	rtc.setDate(3, 10, 2010);   // Set the date to October 3th, 2010
	
	t = rtc.getTime();
	
	if(t.year != 2010)
	{
		myGLCD.print("Read RTC Error", LEFT, 16*1);
		while(1);
	}
	
	myGLCD.print("RTC test OK", LEFT, 16*1);
	
	
	//Test W5100
	NextFlag = 0;
	myGLCD.print("Press the screen to start ping Test", LEFT, 16*7);
	Serial.print("Press the screen to start ping Test");
	myGLCD.clrScr();
	while(!NextFlag)
	{
		if (Touch_DataAvailable())
		{
		Touch_Read();
		i=Touch_GetX();
		j=Touch_GetY();
		if((i > 0) && (i < 239) && (j > 0) && (j < 319))
			NextFlag = 1;
		}
	}
	//reset the w5100
	pinMode(47,OUTPUT);
	digitalWrite(47,LOW);
	delay(500);
	digitalWrite(47,HIGH);
	delay(50);
	pinMode(47,INPUT);
	Ethernet.begin(mac, ip);
	
	for(i = 0;i<10;i++)
	{
		ICMPPing ping(pingSocket);
		j = ping(4, pingAddr, buffer);
		if(j)
			myGLCD.print("Ping 10.0.0.20..OK", LEFT, 16*(i+2));	
		else
			myGLCD.print("Ping 10.0.0.20..Error", LEFT, 16*(i+2));
		
		delay(500);
	}
	myGLCD.clrScr();
	myGLCD.print("Finish test", LEFT, 16);	
  
}

void loop()
{}

void Touch_Init(void)
{
	pinMode(T_CLK,  OUTPUT);
    pinMode(T_CS,   OUTPUT);
    pinMode(T_DIN,  OUTPUT);
    pinMode(T_DOUT, INPUT);
    pinMode(T_IRQ,  INPUT);

	digitalWrite(T_CS,  HIGH);
	digitalWrite(T_CLK, HIGH);
	digitalWrite(T_DIN, HIGH);
	digitalWrite(T_CLK, HIGH);
}

void Touch_WriteData(unsigned char data)
{
	unsigned char temp;
	unsigned char nop;
	unsigned char count;

	temp=data;
	digitalWrite(T_CLK,LOW);

	for(count=0; count<8; count++)
	{
		if(temp & 0x80)
			digitalWrite(T_DIN, HIGH);
		else
			digitalWrite(T_DIN, LOW);
		temp = temp << 1; 
		digitalWrite(T_CLK, LOW);                
		nop++;
		digitalWrite(T_CLK, HIGH);
		nop++;
	}
}

unsigned int Touch_ReadData()
{
	unsigned char nop;
	unsigned int data = 0;
	unsigned char count;
	for(count=0; count<12; count++)
	{
		data <<= 1;
		digitalWrite(T_CLK, HIGH);               
		nop++;
		digitalWrite(T_CLK, LOW);
		nop++;
		if (digitalRead(T_DOUT))
			data++;
	}
	return(data);
}
void Touch_Read()
{
	unsigned long tx=0;
	unsigned long ty=0;

	digitalWrite(T_CS,LOW);                    

	for (int i=0; i<PREC_TOUCH_CONST; i++)
	{
		Touch_WriteData(0x90);        
		digitalWrite(T_CLK,HIGH);
		digitalWrite(T_CLK,LOW); 
		ty+=Touch_ReadData();

		Touch_WriteData(0xD0);      
		digitalWrite(T_CLK,HIGH);
		digitalWrite(T_CLK,LOW);
		tx+=Touch_ReadData();
	}

	digitalWrite(T_CS,HIGH);

	TP_X=tx/PREC_TOUCH_CONST;
	TP_Y=ty/PREC_TOUCH_CONST;
}

char Touch_DataAvailable()
{
  char avail;
  avail = !digitalRead(T_IRQ);
  return avail;
}

int Touch_GetX()
{
	int value;
	value = ((TP_X-PixOffsX)/PixSizeX);
	if (value < 0)
		value = 0;
	return value;
}
int Touch_GetY()
{
	int value;
	value = ((TP_Y-PixOffsY)/PixSizeY);
	if (value < 0)
		value = 0;
	return value;
}